@@ -545,11 +545,8 @@ if (is.null(X) & (!is.null(Z))) { Z <- as.matrix(Z);  X <- matrix(1,nrow(Z),1)}
 if (is.null(Z) & (!is.null(X)))  {X <- as.matrix(X);  Z <- matrix(0,nrow(X),1); gamma <- 0}
 if (is.null(X)) X <- 1;
 
-if (is.null(X2)) X2 <- X;
-if (is.null(Z2)) Z2 <- Z2;
-if (is.null(Z2) & (!is.null(object$gamma))) Z2 <- Z
-if (is.null(X2) & (!is.null(Z2)))  {Z2 <- as.matrix(Z2);  X2 <- matrix(1,nrow(Z2),1)}
-if (is.null(Z2) & (!is.null(X2)))  {X2 <- as.matrix(X2);  Z2 <- matrix(0,nrow(X2),1); gamma <- 0}
+X2 <- X;
+Z2 <- Z2;
 
 if (diag==FALSE) {
    time.part <-  X %*% t(cum[,-1]) 
