@@ -6,61 +6,34 @@
 void Gtranssurv(times,Ntimes,designX,nx,px,designG,ng,pg,antpers,start,stop,
 betaS,Nit,cu,vcu,loglike,Iinv,Vbeta,detail,sim,antsim,
 rani,Rvcu,RVbeta,test,testOBS,Ut,simUt,Uit,id,status,wscore,
-score,dhatMit,dhatMitiid,retur,exppar,sym,mlestart)
+score,dhatMit,dhatMitiid,retur,exppar,sym,mlestart,stratum)
 double *designX,*designG,*times,*betaS,*start,*stop,*cu,*loglike,*Vbeta,*RVbeta,
 *vcu,*Rvcu,*Iinv,*test,*testOBS,*Ut,*simUt,*Uit,*score,*dhatMit,*dhatMitiid;
 int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status,
-*wscore,*retur,*exppar,*sym,*mlestart;
+*wscore,*retur,*exppar,*sym,*mlestart,*stratum;
 {
+// {{{
   matrix *ldesignX,*cdesG,*ldesignG,*cdesX,*cdesX2,*cdesX3,*cdesX4,*CtVUCt,*A,*AI;
   matrix *dYI,*Ct,*dM1M2,*M1M2t,*COV,*ZX,*ddesG,*ZP,*ZPX; 
-  matrix *tmp1,*tmp2,*tmp5,*tmp3,*dS,*S1,*SI,*S2,*M1,*VU,*ZXAI,*VUI, *tmp6; // Added tmp6 
+  matrix *tmp1,*tmp2,*tmp5,*tmp3,*dS,*S1,*SI,*S2,*M1,*VU,*VUI, *tmp6; // Added tmp6 
   matrix *RobVbeta,*Delta,*tmpM1,*Utt,*Delta2,*tmpM2;
   matrix *St[*Ntimes],*M1M2[*Ntimes],*C[*Ntimes],*ZXAIs[*Ntimes],*dYIt[*Ntimes];
   matrix *dW3t[*antpers],*W3t[*antpers],*W4t[*antpers],*W2t[*antpers],*AIxit[*antpers],*Uti[*antpers],*tmp4,*Fst[(*Ntimes)*(*Ntimes)]; 
   matrix *dG[*Ntimes],*cumdG,*Ft[*Ntimes],*ZcX2AIs[*Ntimes],*ZcX2[*Ntimes],*S0tI[*Ntimes],*Ident,*gt[*Ntimes],*q2t[*Ntimes],*G1mG2t[*Ntimes],*q1t[*antpers]; 
-  vector *dLamt[*antpers]; 
-  vector *dA,*VdA,*dN,*MdA,*delta,*zav,*lamt,*plamt,*dlamt;
-  vector *xi,*zi,*U,*beta,*xtilde; 
-  vector *Gbeta,*zcol,*one,*difzzav; 
+  vector *dA,*VdA,*MdA,*delta,*zav,*lamt,*plamt,*dlamt;
+  vector *xi,*zi,*U,*beta,*xtilde,*Gbeta,*zcol,*one,*difzzav; 
   vector *offset,*weight,*ZXdA[*Ntimes],*varUthat[*Ntimes],*Uprofile;
-  vector *Lplamt,*ta,*ahatt,*risk; 
+  vector *ta,*ahatt,*risk; 
   vector *tmpv1,*tmpv2,*rowX,*rowZ,*difX,*VdB,*lht; 
-  vector *W2[*antpers],*W3[*antpers],*reszpbeta,*res1dim,*dAt[*Ntimes]; 
+  vector *dLamt[*antpers],*dAt[*Ntimes]; 
+  vector *W2[*antpers],*W3[*antpers],*reszpbeta,*res1dim;
   int t,c,robust=1,pers=0,i,j,k,l,s,it,count,pmax; 
   int *ipers=calloc(*Ntimes,sizeof(int)); 
   double time=0,dummy,ll; 
-  double tau,hati=0,random,sumscore; 
+  double tau,dhati,hati=0,random,sumscore; 
   double norm_rand(); 
   void GetRNGstate(),PutRNGstate();  
 
-  for (j=0;j<*antpers;j++) { 
-    malloc_vec(*Ntimes,dLamt[j]); malloc_mat(*Ntimes,*px,W3t[j]);
-    malloc_mat(*Ntimes,*px,dW3t[j]); malloc_mat(*Ntimes,*px,W4t[j]);
-    malloc_mat(*Ntimes,*pg,W2t[j]); malloc_mat(*Ntimes,*pg,Uti[j]);
-    malloc_vec(*pg,W2[j]); malloc_vec(*px,W3[j]);
-    malloc_mat(*Ntimes,*pg,q1t[j]); malloc_mat(*Ntimes,*px,AIxit[j]);
-  }
-  malloc_mat(*Ntimes,*px,Delta); malloc_mat(*Ntimes,*px,tmpM1);
-  malloc_mat(*Ntimes,*pg,Delta2); malloc_mat(*Ntimes,*pg,tmpM2); malloc_mat(*Ntimes,*pg,Utt);
-  malloc_vec(1,reszpbeta); malloc_vec(1,res1dim);
-
-  malloc_vec(*Ntimes,lht);
-  malloc_mats(*antpers,*px,&ldesignX,&cdesX,&cdesX2,&cdesX3,&cdesX4,NULL);
-  malloc_mats(*antpers,*pg,&ZP,&cdesG,&ldesignG,&ddesG,NULL); 
-  malloc_mats(*px,*px,&tmp4,&Ident,&COV,&A,&AI,&M1,&CtVUCt,NULL); 
-  malloc_mats(*pg,*pg,&RobVbeta,&tmp1,&tmp2,&dS,&S1,&S2,&SI,&VU,&VUI,NULL); 
-  malloc_mats(*pg,*px,&ZXAI,&tmp5,&tmp3,&ZX,&dM1M2,&M1M2t,NULL); // changed dims of tmp3 and tmp5 
-  malloc_mats(*px,*pg,&cumdG,&ZPX,&dYI,&Ct,NULL); // changed dims of tmp3 and tmp5
-  malloc_mat(*px,*pg,tmp6);
-
-  malloc_vecs(*antpers,&Lplamt,&risk,&weight,&dlamt,&plamt,&lamt,&dN,&zcol,&Gbeta,&one,&offset,NULL); 
-  malloc_vecs(*px,&ahatt,&tmpv1,&difX,&VdB,&rowX,&xi,&dA,&VdA,&MdA,NULL); 
-  malloc_vecs(*px,&ta,&xtilde,NULL); 
-  malloc_vecs(*pg,&tmpv2,&rowZ,&zi,&U,&beta,&delta,&zav,&difzzav,&Uprofile,NULL); 
-
-  identity_matrix(Ident);
-
   for(j=0;j<*Ntimes;j++) {
     malloc_mat(*px,*px,Ft[j]); malloc_mat(*pg,*px,ZcX2AIs[j]); malloc_mat(*pg,*px,gt[j]);
     malloc_mat(*pg,*px,G1mG2t[j]); malloc_mat(*pg,*px,q2t[j]); malloc_mat(*pg,*px,ZcX2[j]);
@@ -71,20 +44,52 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
     for(i=0;i<=j;i++){ malloc_mat(*px,*px,Fst[j*(*Ntimes)+i]); }
   }
 
+
+  for (j=0;j<*antpers;j++) { 
+    malloc_vec(*Ntimes,dLamt[j]); malloc_mat(*Ntimes,*px,W3t[j]);
+    malloc_mat(*Ntimes,*px,dW3t[j]); malloc_mat(*Ntimes,*px,W4t[j]);
+    malloc_mat(*Ntimes,*pg,W2t[j]); malloc_mat(*Ntimes,*pg,Uti[j]);
+    malloc_vec(*pg,W2[j]); malloc_vec(*px,W3[j]);
+    malloc_mat(*Ntimes,*pg,q1t[j]); malloc_mat(*Ntimes,*px,AIxit[j]);
+  }
+
+  malloc_mat(*Ntimes,*px,Delta); malloc_mat(*Ntimes,*px,tmpM1);
+  malloc_mat(*Ntimes,*pg,Delta2); malloc_mat(*Ntimes,*pg,tmpM2); malloc_mat(*Ntimes,*pg,Utt);
+
+
+  malloc_mats(*antpers,*px,&ldesignX,&cdesX,&cdesX2,&cdesX3,&cdesX4,NULL);
+  malloc_mats(*antpers,*pg,&ZP,&cdesG,&ldesignG,&ddesG,NULL); 
+  malloc_mats(*px,*px,&tmp4,&Ident,&COV,&A,&AI,&M1,&CtVUCt,NULL); 
+  malloc_mats(*pg,*pg,&RobVbeta,&tmp1,&tmp2,&dS,&S1,&S2,&SI,&VU,&VUI,NULL); 
+  malloc_mats(*pg,*px,&tmp5,&tmp3,&ZX,&dM1M2,&M1M2t,NULL);
+  malloc_mats(*px,*pg,&cumdG,&ZPX,&dYI,&Ct,NULL); 
+  malloc_mat(*px,*pg,tmp6);
+
+  malloc_vec(1,reszpbeta); malloc_vec(1,res1dim);
+  malloc_vec(*Ntimes,lht);
+  malloc_vecs(*antpers,&risk,&weight,&dlamt,&plamt,&lamt,&zcol,&Gbeta,&one,&offset,NULL); 
+  malloc_vecs(*px,&ahatt,&tmpv1,&difX,&VdB,&rowX,&xi,&dA,&VdA,&MdA,NULL); 
+  malloc_vecs(*px,&ta,&xtilde,NULL); 
+  malloc_vecs(*pg,&tmpv2,&rowZ,&zi,&U,&beta,&delta,&zav,&difzzav,&Uprofile,NULL); 
+
+  identity_matrix(Ident);
+
 //  if (*px>=*pg){ pmax=*px; } else { pmax=*pg; } 
   pmax=max(*px,*pg); ll=0; vec_ones(one);
   for(j=0;j<*pg;j++){ VE(beta,j)=betaS[j]; }
   vec_ones(difX); cu[0]=times[0]; 
+  // }}}
 
   /* Main procedure ================================== */
   for (it=0;it<*Nit;it++){
     vec_zeros(U); mat_zeros(S1);  sumscore=0; 
 
-    for (s=1;s<*Ntimes;s++){
+    for (s=1;s<*Ntimes;s++){ // {{{
       time=times[s]; 
-      mat_zeros(ldesignX); mat_zeros(ldesignG); vec_zeros(risk); 
+      mat_zeros(ldesignX); mat_zeros(ldesignG); 
+//      vec_zeros(risk); 
 
-      for (c=0,count=0;((c<*nx) && (count!=*antpers));c++) {
+      for (c=0,count=0;((c<*nx) && (count!=*antpers));c++) { // {{{
 	if ((start[c]<time) && (stop[c]>=time))  {
 	  VE(risk,id[c])=1.0;
 	  for(j=0;j<pmax;j++) {
@@ -96,8 +101,7 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	  } 
 	  count=count+1; 
 	}
-      }
-
+      } // }}}
 
 //readXZtsimple(antpers,nx,px,designX,pg,designG,
 //		start,stop,status,pers, ldesignX, ldesignG,time, s,id); 
@@ -134,14 +138,24 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	vec_add(tmpv2,rowZ,rowZ); 
 	scl_vec_mult(VE(dlamt,j),rowZ,rowZ); 
 	replace_row(ddesG,j,rowZ); 
+
+	if (it<0) { // use previous hazard estimate
+           dhati=vec_prod(xi,dAt[s]); 
+	   scl_vec_mult(dhati,rowZ,rowZ); 
+	   replace_row(ZP,j,rowZ);
+	}
       }
 
       if (s < 0) {
 	print_vec(plamt); print_vec(dlamt); print_mat(cdesX); 
       }
 
-      MtA(cdesX,ldesignX,A); 
-      invert(A,AI); 
+     MtA(cdesX,ldesignX,A); 
+     invertS(A,AI,1); 
+    if (ME(AI,0,0)==0.0 && *detail==1 && *stratum==1){ 
+       Rprintf(" X'X not invertible at time %lf \n",time); }
+    if (*stratum==1)  { for (k=0;k<*px;k++) if (fabs(ME(A,k,k))<0.000001)  ME(AI,k,k)=0; else ME(AI,k,k)=1/ME(A,k,k);  }
+
       scl_mat_mult(1.0,AI,S0tI[s]); 
       extract_row(ldesignX,pers,xi); 
       Mv(AI,xi,dA); 
@@ -166,12 +180,10 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
       vec_add(difzzav,U,U); 
 
       Mv(ldesignX,dA,lamt);  
+      if (it>=0)  // first time use update hazard est
       for (j=0;j<*antpers;j++){
+	if (s<0 && j<5 ) { Rprintf(" %ld %ld \n",(long int) s, (long int)j); print_vec(zi); }
 	extract_row(ddesG,j,zi); 
-	if (s<0 && j<5 ) {
-	  Rprintf(" %ld %ld \n",(long int) s, (long int)j); 
-	  print_vec(zi); 
-	}
 	scl_vec_mult(VE(lamt,j),zi,zi); 
 	replace_row(ZP,j,zi);
       }
@@ -186,7 +198,6 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	print_mat(tmp3); 
 	print_mat(dG[s]); 
       }
-
       MxA(ZXAIs[s],ZPX,SI); 
       mat_transp(SI,tmp2); 
       MtA(ldesignG,ZP,tmp1); 
@@ -211,11 +222,10 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
       scl_mat_mult(1.0,S1,St[s]); 
 
       /* variance and other things */ 
-      if (it==((*Nit)-1)) { 
+      if (it==((*Nit)-1)) {  // {{{
 	replace_row(Utt,s,U);
 
-
-	for (j=0;j<*px;j++)  {
+	for (j=0;j<*px;j++)  { // {{{
 	  for (i=0;i<*antpers;i++){
 	    dummy=ME(ldesignX,i,j); 
 	    extract_row(cdesX2,i,xi); 
@@ -228,10 +238,8 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	  for (k=0;k<*px;k++){
 	    ME(Ft[s],j,k)=VE(xi,k); 
 	  }
-
 	  VE(lht,s)=VE(lht,s-1)-ME(A,0,0)*(ME(AI,0,0)*ME(AI,0,0));
 
-
 	  /* Rprintf(" %ld %lf %lf \n",s,lht->ve[s],AI->me[0][0]); */
 
 	  MtA(ldesignG,cdesX3,ZcX2[s]); 
@@ -244,8 +252,7 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	  for (k=0;k<*pg;k++) {
 	    ME(G1mG2t[s],k,j)=ME(G1mG2t[s],k,j)+VE(zi,k); 
 	  }
-	}
-
+	} // }}}
 
 	/*
 	  for (i=0;i<*px;i++){ for (j=0;j<*pg;j++) dM1M2->me[j][i]=dA->ve[i]*difzzav->ve[j];
@@ -269,15 +276,15 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	  replace_row(q1t[j],s,zi); 
 	  VE(dLamt[j],s)=VE(plamt,j)*vec_sum(vec_star(xi,dA,rowX));
 	}
-      }/* if (it==((*Nit)-1)) */
+      } // }}} /* if (it==((*Nit)-1)) */
 
-    } /* Ntimes */ 
+    }  // }}} /* Ntimes */ 
 
-    invert(S1,SI); 
+    invertS(S1,SI,1); 
     Mv(SI,U,delta); 
     vec_add(beta,delta,beta); 
 
-    if (*detail>=1) { 
+    if (*detail>=1) { // {{{
       Rprintf("====================Iteration %ld ==================== \n",(long int) it);
       Rprintf("delta \n"); 
       print_vec(delta); 
@@ -289,23 +296,17 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
       print_mat(SI); 
       Rprintf("simple D2 l\n");  
       print_mat(S1); 
-    }
+    } // }}}
 
-    for (k=0;k<*pg;k++) {
-      sumscore += VE(U,k); 
-    }
-
-    if ((fabs(sumscore)<0.000001) & (it<*Nit-2)){ 
-      it=*Nit-2; 
-    }
+    for (k=0;k<*pg;k++) sumscore += VE(U,k); 
+    if ((fabs(sumscore)<0.000001) & (it<*Nit-2)) it=*Nit-2; 
+    
   } /* it */
-  for (k=0;k<*pg;k++) { 
-    score[k]=VE(U,k); 
-  }
+  for (k=0;k<*pg;k++) score[k]=VE(U,k); 
 
 
   /* computation of q(t) */
-  for (s=1;s<*Ntimes;s++) {
+  for (s=1;s<*Ntimes;s++) { // {{{
     mat_zeros(M1M2t); 
 
     for (t=s;t<*Ntimes;t++) { 
@@ -338,11 +339,11 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
         print_mat(mat_transp(q2t[s],tempTranspose));
 	free_mat(tempTranspose);
     }
-  }
+  } // }}}
 
   /* terms for robust variances ============================ */
-  if (robust==1) {
-    for (s=1;s<*Ntimes;s++) {
+  if (robust==1) { // {{{
+    for (s=1;s<*Ntimes;s++) { // {{{
       time=times[s]; 
       cu[s]=times[s]; vcu[s]=times[s]; Rvcu[s]=times[s]; Ut[s]=times[s]; 
 
@@ -413,13 +414,13 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
 	}
       } /* i=1..antpers */ 
 
-    } /* s=1 ..Ntimes */ 
+    } // }}} /* s=1 ..Ntimes */ 
 
     MxA(SI,VU,S2); 
     MxA(S2,SI,VU); 
 
     /* ROBUST VARIANCES   */
-    for (s=1;s<*Ntimes;s++) {
+    for (s=1;s<*Ntimes;s++) { // {{{
 
       if (s<0){ 
 	print_mat(dG[s]); 
@@ -503,10 +504,10 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
       for (k=1;k<*px+1;k++) { 
 	Rvcu[k*(*Ntimes)+s]=VE(VdB,k-1); vcu[k*(*Ntimes)+s]=VE(VdB,k-1); 
       }
-    }  /*  s=1 ..Ntimes */ 
+    } // }}} /*  s=1 ..Ntimes */ 
     MxA(RobVbeta,SI,tmp1); 
     MxA(SI,tmp1,RobVbeta);
-  }
+  } // }}}
 
   for(j=0;j<*pg;j++) { 
     betaS[j]= VE(beta,j); loglike[0]=ll;
@@ -516,7 +517,7 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
     } 
   } 
 
-  if (*sim==1) {
+  if (*sim==1) { // {{{
     Rprintf("Simulations start N= %ld \n",(long int) *antsim);
     GetRNGstate();  /* to use R random normals */
 
@@ -623,33 +624,36 @@ int *nx,*px,*ng,*pg,*antpers,*Ntimes,*Nit,*detail,*sim,*antsim,*rani,*id,*status
       }  /* s=1..Ntims */
     }  /* k=1..antsim */
     PutRNGstate();  /* to use R random normals */
-  } /* sim==1 */
+  } // }}} /* sim==1 */
   
+  // {{{ freeing
   free_mats(&cumdG,&tmp4,&Ident,&ddesG,&Utt,&tmpM2,&VUI,&ZX,&COV,
-		&dM1M2,&AI,&A,&ZXAI,&tmp1,&tmp2,&tmp3,&ldesignX,&cdesX,
+		&dM1M2,&AI,&A,&tmp1,&tmp2,&tmp5,&tmp3,&ldesignX,&cdesX,
 		&cdesX2,&cdesX4,&cdesX3,&cdesG,&ldesignG,&M1,&dS,&S1,&SI,NULL);
-  free_mats(&S2,&VU,&ZP,&ZPX,&dYI,&Ct,&M1M2t,&RobVbeta,&Delta,&Delta2,
+  free_mats(&tmp6,&S2,&VU,&ZP,&ZPX,&dYI,&Ct,&M1M2t,&RobVbeta,&Delta,&Delta2,
 		&tmpM1,&CtVUCt,NULL); 
 
-  free_vecs(&risk,&ta,&ahatt,&Uprofile,&dlamt,&plamt,&lamt,&one,&xi,&zcol,&Gbeta,&VdA,&dA,&MdA,&xtilde,&zi,&U,&beta,&delta,&zav,&difzzav,&weight,&offset,&tmpv1,&tmpv2,&rowX,&rowZ,&difX,&VdB,&reszpbeta,&res1dim,NULL); 
-
-
-  free_mat(tmp6);
+  free_vecs(&lht,&risk,&ta,&ahatt,&Uprofile,&dlamt,&plamt,&lamt,&one,&xi,&zcol,&Gbeta,&VdA,&dA,&MdA,&xtilde,&zi,&U,&beta,&delta,&zav,&difzzav,&weight,&offset,&tmpv1,&tmpv2,&rowX,&rowZ,&difX,&VdB,&reszpbeta,&res1dim,NULL); 
 
   for (j=0;j<*antpers;j++) {
-    free_mat(W3t[j]); free_mat(dW3t[j]); 
-    free_mat(W4t[j]); free_mat(W2t[j]); free_vec(W2[j]); free_vec(W3[j]);
-    free_mat(AIxit[j]); free_mat(Uti[j]); free_vec(dLamt[j]); free_mat(q1t[j]); 
+    free_vec(dLamt[j]); free_mat(W3t[j]); 
+    free_mat(dW3t[j]); free_mat(W4t[j]); 
+    free_mat(W2t[j]); free_mat(Uti[j]); 
+    free_vec(W2[j]); free_vec(W3[j]);
+    free_mat(q1t[j]); free_mat(AIxit[j]); 
   }
 
   for (j=0;j<*Ntimes;j++) {
-    free_mat(gt[j]); free_mat(G1mG2t[j]); free_mat(q2t[j]);
-    free_mat(Ft[j]); free_mat(ZcX2AIs[j]); free_mat(ZcX2[j]); free_mat(S0tI[j]); 
-    free_mat(dG[j]); free_mat(dYIt[j]); free_vec(dAt[j]); free_mat(C[j]); free_mat(M1M2[j]);
-    free_mat(ZXAIs[j]); free_vec(ZXdA[j]); free_mat(St[j]); free_vec(varUthat[j]); 
+    free_mat(Ft[j]); free_mat(ZcX2AIs[j]); free_mat(gt[j]); 
+    free_mat(G1mG2t[j]); free_mat(q2t[j]); free_mat(ZcX2[j]); 
+    free_mat(S0tI[j]); free_mat(dG[j]); free_mat(C[j]); 
+    free_mat(M1M2[j]); free_mat(ZXAIs[j]); free_mat(dYIt[j]); 
+    free_vec(dAt[j]); free_vec(ZXdA[j]); free_mat(St[j]); 
+    free_vec(varUthat[j]); 
     for(i=0;i<=j;i++) free_mat(Fst[j*(*Ntimes)+i]); 
   } 
 
   free(ipers); 
+  // }}}
 
 }
